@@ -179,6 +179,8 @@ def export_pytorch_ensemble(
         workflow,
         name + "_nvt",
         preprocessing_path,
+        output_model="pytorch",
+        sparse_max=sparse_max,
         backend=nvtabular_backend,
         cats=cats,
         conts=conts,
@@ -313,7 +315,7 @@ def _generate_ensemble_config(name, output_path, nvt_config, nn_config, name_ext
     config.ensemble_scheduling.step.append(nvt_step)
     config.ensemble_scheduling.step.append(tf_step)
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
@@ -429,30 +431,15 @@ def _generate_nvtabular_config(
         config.output.append(
             model_config.ModelOutput(name="ROWINDEX", data_type=model_config.TYPE_INT32, dims=[-1])
         )
-    elif output_model == "pytorch":
-        for col_name, col_schema in workflow.input_schema.column_schemas.items():
-            _add_model_param(col_schema, model_config.ModelInput, config.input)
-
-        for col_name, col_schema in workflow.output_schema.column_schemas.items():
-            _add_model_param(
-                col_schema,
-                model_config.ModelOutput,
-                config.output,
-                [-1, 1],
-            )
     else:
         for col_name, col_schema in workflow.input_schema.column_schemas.items():
             _add_model_param(col_schema, model_config.ModelInput, config.input)
 
         for col_name, col_schema in workflow.output_schema.column_schemas.items():
-            if sparse_max and col_name in sparse_max.keys():
-                # this assumes max_sequence_length is equal for all output columns
-                dim = sparse_max[col_name]
-                _add_model_param(col_schema, model_config.ModelOutput, config.output, [-1, dim])
-            else:
-                _add_model_param(col_schema, model_config.ModelOutput, config.output)
+            dim = sparse_max[col_name] if sparse_max and col_name in sparse_max.keys() else 1
+            _add_model_param(col_schema, model_config.ModelOutput, config.output, [-1, dim])
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
@@ -510,7 +497,7 @@ def export_tensorflow_model(model, name, output_path, version=1):
             )
         )
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
@@ -560,7 +547,8 @@ def export_pytorch_model(
     config = model_config.ModelConfig(name=name, backend=backend)
 
     for col_name, col_schema in workflow.output_schema.column_schemas.items():
-        _add_model_param(col_schema, model_config.ModelInput, config.input)
+        dim = sparse_max[col_name] if sparse_max and col_name in sparse_max.keys() else 1
+        _add_model_param(col_schema, model_config.ModelInput, config.input, [-1, dim])
 
     *_, last_layer = model.parameters()
     dims = last_layer.shape[0]
@@ -572,13 +560,15 @@ def export_pytorch_model(
     )
 
     if sparse_max:
-        with open(os.path.join(output_path, str(version), "model_info.json"), "w") as o:
-            model_info = dict()
+        with open(
+            os.path.join(output_path, str(version), "model_info.json"), "w", encoding="utf8"
+        ) as o:
+            model_info = {}
             model_info["sparse_max"] = sparse_max
             model_info["use_fix_dtypes"] = use_fix_dtypes
             json.dump(model_info, o)
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
@@ -600,7 +590,7 @@ def _generate_pytorch_config(model, name, output_path, max_batch_size=None):
             )
         )
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
@@ -671,7 +661,7 @@ def _generate_hugectr_config(name, output_path, hugectr_params, max_batch_size=N
     embeddingkey_long_type = model_config.ModelParameter(string_value=embeddingkey_long_type_val)
     config.parameters["embeddingkey_long_type"].CopyFrom(embeddingkey_long_type)
 
-    with open(os.path.join(output_path, "config.pbtxt"), "w") as o:
+    with open(os.path.join(output_path, "config.pbtxt"), "w", encoding="utf8") as o:
         text_format.PrintMessage(config, o)
     return config
 
