@@ -14,30 +14,47 @@
 # limitations under the License.
 #
 import contextlib
+import logging
 import os
 
+import dask.dataframe as dd
 import numpy as np
-import tensorflow as tf
 
-from nvtabular.io.dataset import Dataset
+from merlin.core.dispatch import HAS_GPU
+from merlin.schema import Tags
 from nvtabular.loader.backend import DataLoader
 from nvtabular.loader.tf_utils import configure_tensorflow, get_dataset_schema_from_feature_columns
-from nvtabular.ops import _get_embedding_order
-from nvtabular.tags import Tags
 
 from_dlpack = configure_tensorflow()
+LOG = logging.getLogger("nvtabular")
+# tf import must happen after config to restrict memory use
+import tensorflow as tf  # noqa
 
+# noqa
+try:
+    from merlin.io import Dataset
+
+    nvt_dataset_class = Dataset
+except ImportError:
+    nvt_dataset_class = None
 # pylint has issues with TF array ops, so disable checks until fixed:
 # https://github.com/PyCQA/pylint/issues/3613
 # pylint: disable=no-value-for-parameter,unexpected-keyword-arg,redundant-keyword-arg
 
 
-def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, reader_kwargs):
+dd_engine = {
+    "parquet": dd.read_parquet,
+    "csv": dd.read_csv,
+    "df": dd.DataFrame,
+}
+
+
+def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, device, reader_kwargs):
     # TODO: put this in parent class and allow
     # torch dataset to leverage as well?
 
     # if a dataset was passed, just return it
-    if isinstance(paths_or_dataset, Dataset):
+    if hasattr(paths_or_dataset, "schema"):
         return paths_or_dataset
 
     # otherwise initialize a dataset
@@ -56,27 +73,28 @@ def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, reader_
     if len(files) == 0:
         raise ValueError(_is_empty_msg)
 
-    # implement buffer size logic
-    # TODO: IMPORTANT
-    # should we divide everything by 3 to account
-    # for extra copies laying around due to asynchronicity?
-    reader_kwargs = reader_kwargs or {}
-    if buffer_size >= 1:
-        if buffer_size < batch_size:
-            reader_kwargs["batch_size"] = int(batch_size * buffer_size)
-        else:
-            reader_kwargs["batch_size"] = buffer_size
+    if not engine:
+        # default engine is parquet
+        engine = "parquet"
+
+    cpu = device and "cpu" in device
+
+    if nvt_dataset_class:
+        return nvt_dataset_class(files, engine=engine, cpu=cpu)
     else:
-        reader_kwargs["part_mem_fraction"] = buffer_size
-    return Dataset(files, engine=engine, **reader_kwargs)
+        LOG.warning(
+            "NVTabular Dataset class not detected, reverting to Dask Dataframe."
+            "Expect slower iteration speeds."
+        )
+    return dd_engine[engine](files)
 
 
 def _validate_schema(feature_columns, cat_names, cont_names, schema=None):
     _uses_feature_columns = feature_columns is not None
     _uses_explicit_schema = (cat_names is not None) or (cont_names is not None)
 
-    cat_tag_names = schema.select_by_tag([Tags.CATEGORICAL]).column_names
-    cont_tag_names = schema.select_by_tag([Tags.CONTINUOUS]).column_names
+    cat_tag_names = schema.select_by_tag([Tags.CATEGORICAL]).column_names if schema else []
+    cont_tag_names = schema.select_by_tag([Tags.CONTINUOUS]).column_names if schema else []
     _uses_dataset_schema = cat_tag_names or cont_tag_names
 
     if _uses_feature_columns and _uses_explicit_schema:
@@ -100,6 +118,12 @@ def _validate_schema(feature_columns, cat_names, cont_names, schema=None):
         )
 
 
+def _get_schema(dataset):
+    if hasattr(dataset, "schema"):
+        return dataset.schema
+    return None
+
+
 class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
     """
     Infinite generator used to asynchronously iterate through CSV or Parquet
@@ -153,34 +177,34 @@ class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
 
     Parameters
     -------------
-    - paths_or_dataset: str or list(str)
+    paths_or_dataset: str or list(str)
         Either a string representing a file pattern (see `tf.glob` for
         pattern rules), a list of filenames to be iterated through, or
         a Dataset object, in which case `buffer_size`, `engine`, and
         `reader_kwargs` will be ignored
-    - batch_size: int
+    batch_size: int
         Number of samples to yield at each iteration
-    - label_names: list(str)
+    label_names: list(str)
         Column name of the target variable in the dataframe specified by
         `paths_or_dataset`
-    - feature_columns: list(tf.feature_column) or None
+    feature_columns: list(tf.feature_column) or None
         A list of TensorFlow feature columns representing the inputs
         exposed to the model to be trained. Columns with parent columns
         will climb the parent tree, and the names of the columns in the
         unique set of terminal columns will be used as the column names.
         If left as None, must specify `cat_names` and `cont_names`
-    - cat_names: list(str) or None
+    cat_names: list(str) or None
         List of categorical column names. Ignored if `feature_columns` is
         specified
-    - cont_names: list(str) or None
+    cont_names: list(str) or None
         List of continuous column names. Ignored if `feature_columns` is
         specified
-    - engine: {'csv', 'parquet', None}, default None
+    engine: {'csv', 'parquet', None}, default None
         String specifying the type of read engine to use. If left as `None`,
         will try to infer the engine type from the file extension.
-    - shuffle: bool, default True
+    shuffle: bool, default True
         Whether to shuffle chunks of batches before iterating through them.
-    - buffer_size: float or int
+    buffer_size: float or int
         If `0 <  buffer_size < 1`, `buffer_size` will refer to the fraction of
         total GPU memory to occupy with a buffered chunk. If `1 < buffer_size <
         batch_size`, the number of rows read for a buffered chunk will
@@ -189,20 +213,20 @@ class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
         the last chunk in a dataset, which will, in general, be smaller).
         Larger chunk sizes will lead to more efficiency and randomness,
         but require more memory.
-    - device: None
+    device: None
         Which GPU device to load from. Ignored for now
-    - parts_per_chunk: int
+    parts_per_chunk: int
         Number of dataset partitions with size dictated by `buffer_size`
         to load and concatenate asynchronously. More partitions leads to
         better epoch-level randomness but can negatively impact throughput
-    - reader_kwargs: dict
+    reader_kwargs: dict
         extra kwargs to pass when instantiating the underlying
         `nvtabular.Dataset`
     sparse_list : list(str) or None
         list with column names of columns that should be represented as sparse tensors
     sparse_max : dict
         dictionary of key: column_name + value: integer representing max sequence length for column
-    sparse_dense : bool
+    sparse_as_dense : bool
         bool value to activate transforming sparse tensors to dense
     """
 
@@ -229,20 +253,18 @@ class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
         sparse_names=None,
         sparse_max=None,
         sparse_as_dense=False,
+        schema=None,
     ):
-        dataset = _validate_dataset(
-            paths_or_dataset, batch_size, buffer_size, engine, reader_kwargs
-        )
-        cat_names, cont_names = _validate_schema(
-            feature_columns, cat_names, cont_names, schema=dataset.schema
-        )
-
-        # sort the columns to avoid getting incorrect output
-        # (https://github.com/NVIDIA/NVTabular/issues/412)
-        cat_names = _get_embedding_order(cat_names)
-        cont_names = _get_embedding_order(cont_names)
-
         device = device or 0
+        device = "cpu" if not HAS_GPU else device
+        dataset = _validate_dataset(
+            paths_or_dataset, batch_size, buffer_size, engine, device, reader_kwargs
+        )
+        schema = _get_schema(dataset) if not schema else schema
+        cat_names, cont_names = _validate_schema(
+            feature_columns, cat_names, cont_names, schema=schema
+        )
+
         DataLoader.__init__(
             self,
             dataset,
@@ -327,10 +349,13 @@ class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
     def _pack(self, gdf):
         if isinstance(gdf, np.ndarray):
             return gdf
-        if hasattr(gdf, "to_dlpack") and callable(getattr(gdf, "to_dlpack")):
+        elif hasattr(gdf, "to_dlpack") and callable(getattr(gdf, "to_dlpack")):
             return gdf.to_dlpack()
         elif hasattr(gdf, "to_numpy") and callable(getattr(gdf, "to_numpy")):
-            return gdf.to_numpy()
+            gdf = gdf.to_numpy()
+            if isinstance(gdf[0], list):
+                gdf = np.stack(gdf)
+            return gdf
         return gdf.toDlpack()
 
     def _unpack(self, gdf):
@@ -350,16 +375,16 @@ class KerasSequenceLoader(tf.keras.utils.Sequence, DataLoader):
             dlpack = self._pack(gdf.values[0])
         else:
             dlpack = self._pack(gdf.values.T)
-
         # catch error caused by tf eager context
         # not being initialized
+
         try:
             x = self._unpack(dlpack)
         except AssertionError:
             tf.random.uniform((1,))
             x = self._unpack(dlpack)
-
-        if gdf.shape[0] == 1:
+        # if rank is already two it is  already in list format
+        if gdf.shape[0] == 1 and not tf.rank(x) == 2:
             # batch size 1 so got squashed to a vector
             x = tf.expand_dims(x, 0)
         elif len(gdf.shape) == 1 or len(x.shape) == 1:
