@@ -19,8 +19,7 @@ from dask.dataframe.utils import meta_nonempty
 
 from merlin.core.dispatch import DataFrameType, annotate
 from merlin.schema import Schema
-
-from .operator import ColumnSelector, Operator
+from nvtabular.ops.operator import ColumnSelector, Operator
 
 
 class Groupby(Operator):
@@ -180,6 +179,10 @@ class Groupby(Operator):
 
         return column_mapping
 
+    @property
+    def dependencies(self):
+        return self.groupby_cols
+
     def _compute_dtype(self, col_schema, input_schema):
         col_schema = super()._compute_dtype(col_schema, input_schema)
 
@@ -193,6 +196,7 @@ class Groupby(Operator):
             "var": numpy.float32,
             "std": numpy.float32,
             "median": numpy.float32,
+            "sum": numpy.float32,
         }
 
         is_lists = {"list": True}
@@ -244,9 +248,9 @@ def _apply_aggs(_df, groupby_cols, _list_aggs, _conv_aggs, name_sep="_", ascendi
             df.drop(columns=[col + f"{name_sep}list"], inplace=True)
 
     for col in df.columns:
-        if re.search(f"{name_sep}(count|nunique)", col):
+        if re.search(f"{name_sep}(count|nunique)$", col):
             df[col] = df[col].astype(numpy.int32)
-        elif re.search(f"{name_sep}(mean|median|std|var)", col):
+        elif re.search(f"{name_sep}(mean|median|std|var|sum)$", col):
             df[col] = df[col].astype(numpy.float32)
 
     return df
@@ -293,9 +297,9 @@ def _first(x):
     # item in the list
     if hasattr(x, "list"):
         # cuDF-specific behavior
-        offsets = x.list._column.offsets
-        elements = x.list._column.elements
-        return elements[offsets[:-1]]
+        offsets = x.list._column.offsets.values
+        elements = x.list.leaves
+        return elements[offsets[:-1]]._column
     else:
         # cpu/pandas
         return x.apply(lambda y: y[0])
@@ -306,9 +310,9 @@ def _last(x):
     # item in the list
     if hasattr(x, "list"):
         # cuDF-specific behavior
-        offsets = x.list._column.offsets
-        elements = x.list._column.elements
-        return elements[offsets[1:].values - 1]
+        offsets = x.list._column.offsets.values
+        elements = x.list.leaves
+        return elements[offsets[1:] - 1]._column
     else:
         # cpu/pandas
         return x.apply(lambda y: y[-1])
